#ifndef DETECTOBJECTS_H_
#define DETECTOBJECTS_H_

#include "common.h"
#include "adacascadeclass.h"

/**
 * Classify source image pixels according to cluster_means_norm matrix.
 *
 * Return a pair of matrices of the same size as source_image
 * 1st - .first  - timg
 * 2nd - .second - wimg
 */
std::pair<CvMat*, CvMat*> ClassifyImagePixels( CvMat * source_image,
                                               CvMat * cluster_means_norm,
                                               PixClass pixclass );

CvMat ** IntegralBlockHist( CvMat * timg,
                            CvMat * wimg,
                            size_t cluster_number,
                            size_t dx, size_t dy );

typedef std::vector<SubWindowInfo> SubWindows;

//*** x-y dimensions of the minimal block in block histograms
struct HistBlockSize
{
    double dx;
    double dy;
	HistBlockSize(double dxin, double dyin):dx(dxin),dy(dyin){}
};
typedef std::vector<HistBlockSize> HistBlockSizeArray;
typedef std::vector<double> ScaleFctArray;

IMPEXP
SubWindows DetectObjects( IplImage* src,
                          Model const & model, 
                          CvMat * cluster_means,
                          CvMat * cov_matrix,
                          CvMat * cluster_means_norm,
						  int pixclass,
						  HistBlockSizeArray bsarray,
						  ScaleFctArray scarray);

void ClearModelCache(Model const & model);

#endif
